#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Carp 'croak';
use List::Util 'max','min','sum','pairmap';
use Math::BaseCnv 'cnv';
use POSIX 'floor';
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh';

use Math::PlanePath::AlternatePaper 118; # v.118 for xyxy_to_n_either()
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh', 'digit_split_lowtohigh',
  'digit_join_lowtohigh', 'round_down_pow';
use Math::PlanePath::Base::Generic
  'round_nearest';

# use Math::PlanePath;
# *_divrem_mutate = \&Math::PlanePath::_divrem_mutate;
# unshift @INC, '/usr/share/perl5';

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyAlternate ':all';
our @ipc;
$|=1;

# uncomment this to run the ### lines
use Smart::Comments;

# GP-DEFINE  default(strictargs,1);
# GP-DEFINE  read("alternate-defines.gp");

{
  # GRS count distinct sub-sequences, by DFAs
  #
  # A005943 number of subwords of length n in GRS (factor complexity)
  # 1,2,4,8,16,24,36,46,56,64,72,80,88,96,104,112,120,128,
  # 136,144,152,160,168,176,184,192,200,208,216,224,232,240,
  # 248,256,264,272,280
  #
  #   2,4,8,16,24,36,46,56,64,72,80,88,96,104,112,120,128,
  # 136,144,152,160,168,176,184,192,200,208,216,224,232,240,
  # 248,256,264,272,280,288,296,304,312,320,328,
  # recurrence_guess(OEIS_samples("A005943"))
  # #OEIS_samples("A005943") == 37
  # 37 << 6 == 2368
  # initial exceptions [1, 2, 4, 8, 16, 24, 36, 46]
  # 8*n - 8

  my @bit_to_flat = (GRS_flat(1),
                     GRS_flat(-1));
  my @pending = map {$bit_to_flat[$_]->clone->MyFLAT::set_name($_)} 0,1;
  print scalar(@pending),",";
  foreach my $len (2 .. 50) {
    my @new_pending;
    foreach my $f (@pending) {
      my $f_inc = $f->MyFLAT::digits_increment(add => 1);
      foreach my $bit (0,1) {
        my $new_f = $f_inc->intersect($bit_to_flat[$bit]);
        if (! $new_f->is_empty) {
          my $name = $f->{'name'};
          $new_f = $new_f->MyFLAT::set_name("$bit$name")
            ->MyFLAT::minimize (verbose => 0);
          push @new_pending, $new_f;
        }
      }
    }
    @pending = @new_pending;
    print scalar(@pending),",";
  }

  if (@pending < 16) {
    my @strings = map {$_->{'name'}} @pending;
    @strings = sort @strings;
    foreach (@strings) { print $_,"\n"; }
  }
  exit 0;
}

{
  # GRSrunSpred State Machine
  my $d1 = GRS_flat(1)->intersect(GRS_flat(-1)->MyFLAT::digits_increment)
    ->MyFLAT::minimize(verbose=>1);;
  my $d2 = GRS_flat(-1)->intersect(GRS_flat(1)->MyFLAT::digits_increment)
    ->MyFLAT::minimize(verbose=>1);;
  my $one = FLAT::Regex->new ('0* 1')->as_dfa;
  my $f = $d1->union($d2,$one)
    ->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # TurnRunSpred
  require FLAT;
  require MyFLAT;

  my $f_one = FLAT::Regex->new ('0* 1')->as_dfa;
  my $f_0m4 = FLAT::Regex->new ('(0|1)* 00')->as_dfa;
  my $f_1m4 = FLAT::Regex->new ('(0|1)* 01')->as_dfa;
  my $f_2m4 = FLAT::Regex->new ('(0|1)* 10')->as_dfa;
  my $f_3m4 = FLAT::Regex->new ('(0|1)* 11')->as_dfa;
  my $f_even = FLAT::Regex->new ('(0|1)* 0')->as_dfa;
  my $f_odd  = FLAT::Regex->new ('(0|1)* 1')->as_dfa;

  my $TurnLpred_even = TurnLpred_flat()->intersect($f_even);
  my $TurnRpred_even = TurnRpred_flat()->intersect($f_even);

  my $case_0m4 = TurnLpred_flat()->intersect($f_0m4);
  my $case_2m4 = TurnRpred_flat()->intersect($f_2m4);
  my $case_1m4 = TurnRpred_flat()
    ->intersect($f_0m4)
    ->MyFLAT::digits_increment(add => 1);
  my $case_3m4 = TurnLpred_flat()
    ->intersect($f_2m4)
    ->MyFLAT::digits_increment(add => 1);

  my $f = $case_0m4->union($case_1m4,
                           $case_2m4,
                           $case_3m4,
                           $f_one
                          );
  # $f = $f->intersect(TurnLpred_flat());
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize;
  # $case_1m4->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  $f->MyFLAT::view;
  MyFLAT::FLAT_print_tikz($f);

  foreach my $n (0 .. 256) {
    my $n2 = reverse cnv($n,10,2);
    my $by_flat = $f->contains($n2) ? 1 : 0;
    my $by_func = TurnRunSpred($n);
    unless ($by_flat == $by_func) {
      print "n=$n [$n2] by_flat $by_flat by_func $by_func\n";
    }
  }
  exit;
}

{
  # LenDir4 pred
  require FLAT;
  require MyFLAT;

  my $f;
  $f = LenDir4_flat(8, le=>0);
  # $f = Dir4within_flat(0,2);
  # $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse->MyFLAT::minimize;
  $f->{'flow'} = 'west';
  MyFLAT::FLAT_print_tikz($f);
  $f->MyFLAT::view;

  {
    Dir4within_flat(0,0)->is_empty or die;
    Dir4within_flat(0,0)->MyFLAT::optional_leading_0s->is_empty or die;

    # {
    #   my $f = Dir4within_flat(0,1);
    #   MyFLAT::FLAT_show_breadth($f, 4);
    # }

    my $bad = 0;
  OUTER: foreach my $d (0 .. 3) {
      foreach my $len (0 .. 10) {
        my $f = Dir4within_flat($d,$len);
        # $f->MyFLAT::view;
        foreach my $n (0 .. 256) {
          my $n2 = cnv($n,10,2);
          my $by_flat = $f->contains($n2) ? 1 : 0;
          my $by_func = Dir4withinPred($n,$d,$len);
          unless ($by_flat == $by_func) {
            print "d=$d len=$len n=$n [$n2] by_flat $by_flat by_func $by_func\n";
            last OUTER if $bad++ > 10;
          }
        }
      }
    }
    $bad == 0 or die;
  }
  exit;

  sub LenDir4_flat {
    my ($len, %options) = @_;
    require FLAT;
    if ($len < 4) {
      return MyFLAT::bits_of_length_flat(-1);
    }
    # must all four Dir4within_flat()
    my $f = Dir4within_flat(0,$len);
    foreach my $d (1 .. 3) {
      $f = $f->intersect(Dir4within_flat($d,$len))
        ->MyFLAT::minimize;
    }
    if ($options{'le'}) {
      return $f->MyFLAT::set_name("LenDir4 len <= $len");
    } else {
      # $f->MyFLAT::view;
      # LenDir4_flat($len-1, le=>1)->MyFLAT::view;
      # $f->difference(LenDir4_flat($len-1, le=>1))->as_dfa->MyFLAT::view;
      return $f->difference(LenDir4_flat($len-1, le=>1))->as_dfa
        ->MyFLAT::minimize
        ->MyFLAT::set_name('LenDir4 len=$len');
    }
  }
  use Memoize;
  BEGIN { memoize('LenDir4_flat', LIST_CACHE=>'MERGE') }

  sub Dir4withinPred {
    my ($n,$d,$len) = @_;
    foreach (1 .. $len) {
      if (dir($n)%4 == $d) { return 1; }
      $n++;
    }
    return 0;
  }

  sub Dir4within_flat {
    my ($d,$len) = @_;
    if ($len <= 0) {
      return MyFLAT::bits_of_length_flat(-1);
    }
    my $f = dir4_flat($d);
    if ($len == 1) {
      return $f;
    }
    return Dir4within_flat($d,$len-1)
      ->MyFLAT::digits_increment(add => -1)
      ->MyFLAT::optional_leading_0s
      ->union($f)->as_dfa
      ->MyFLAT::set_name('Dir4within d=$d len=$len');
  }
  use Memoize;
  BEGIN { memoize('Dir4within_flat', LIST_CACHE=>'MERGE') }
}



{
  # Epred runs of 8, by DpredSecond

  my $f = DpredSecond_flat();
  for (1 .. 7) {
    $f = DpredSecond_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1)
      ->MyFLAT::set_name('DpredSecondEight');
    # $f->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  }
  # $f = $f->intersect(TurnLpred_flat());
  # $f->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;

  $f->MyFLAT::reverse->MyFLAT::blocks(2)->MyFLAT::minimize->MyFLAT::view;

  require FLAT::Regex;
  my $r = FLAT::Regex->new ('((0|1)* 0 | [])
                             (11)* 1 1 (0(11)*1)* 0 (11)* 1010')
    #                  8     8 7 6          6 5     5    0
    ->as_dfa
    ->MyFLAT::set_name('by regexp');
  MyFLAT::FLAT_check_is_equal($r,$f);

  # base-4  1 (3|2)* 2 3* 22        first 1222
  # GP-Test  to_base4(from_base4(22)+7) == 101
  $r = FLAT::Regex->new ('((0|1)* 0 | [])
                             1 (11|10)* 10 (11)* 1010')
    ->as_dfa
    ->MyFLAT::set_name('by regexp 2');
  MyFLAT::FLAT_check_is_equal($r,$f);
  exit 0;
}
{
  # DpredSecond_flat()->intersect(TurnLpred_flat())
  #   ->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  # DpredSecond_flat()->intersect(TurnRpred_flat())
  #   ->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  DpredFirstInf_flat()
    # ->MyFLAT::blocks(2)
    ->MyFLAT::reverse
    ->MyFLAT::minimize->MyFLAT::view;
  DpredSecond_flat()
    # ->MyFLAT::blocks(2)
    ->MyFLAT::reverse
    ->MyFLAT::minimize->MyFLAT::view;
  DpredInf_flat()
    # ->MyFLAT::blocks(2)
    ->MyFLAT::reverse
    ->MyFLAT::minimize->MyFLAT::view;
  exit 0;
}


{
  # Epred runs of 8

  EpredL_flat()->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  EpredR_flat()->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  my $f = Epred_flat();
  for (1 .. 7) {
    $f = Epred_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1);
  }
  $f->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  exit 0;
}

{
  # gThueMorse powers image

  require Image::Base::GD;
  my $width = 1300;
  my $height = 800;
  my $image = Image::Base::GD->new (-height => $height,
                                    -width  => $width);
  $image->rectangle(0,0, $width-1,$height-1, 'black');

  my $t;
  # $t = [ map {ThueMorse($_)} 0 .. $width-1 ];
  # $t = [ map {$_ && BitAboveLowestOne($_)} 0 .. $width-1 ];  # dragon turn
  $t = [ map {$_ && turn($_)==1} 0 .. $width-1 ];  

  my $pow = $t;
  foreach my $y (0 .. $height-1) {
    foreach my $x (0 .. $#$t) {
      if ($pow->[$x]) { $image->xy($x,$y,'white'); }
    }
    $pow = multiply_vecpol($pow, $t);
  }
  my $filename = '/tmp/x.png';
  $image->save($filename);
  system 'xzgv',$filename;
  exit 0;

  sub multiply_vecpol {
    my ($a, $b) = @_;
    my $ret = [];
    foreach my $i (0 .. $#$a) {
      $ret->[$i] = 0;
      foreach my $j (0 .. $i) {
        $ret->[$i] += $a->[$j] * $b->[$i-$j];
      }
      $ret->[$i] %= 2;
    }
    return $ret;
  }
}



{
  # twinalternate graph diameter count paths
  # DiameterCount 1,1,1,2,4,2,264,2,5348880

  require MyGraphs;
  my $verbose = 0;
  my @values;
  foreach my $k (0 .. 8) {
    # my $graph = twinalternate_Graph($k);
    my $graph = alternate_Graph($k);
    if ($verbose) {
      MyGraphs::Graph_xy_print($graph);
    }

    my $diameter_start = $graph->get_graph_attribute('diameter_start');
    my $diameter_end   = $graph->get_graph_attribute('diameter_end');

    my @diameter_path = $graph->SP_Dijkstra($diameter_start,$diameter_end);
    ### @diameter_path
    my $diameter_num_vertices_by_path = scalar(@diameter_path);
    my $diameter = $graph->diameter;

    my $sptg = $graph->SPT_Dijkstra($diameter_start);
    ### p: $graph->get_vertex_attribute($diameter_end,'p')
    ### p: $sptg ->get_vertex_attribute($diameter_end,'p')
    ### sptg  cyclic: $sptg->is_cyclic
    ### graph cyclic: $graph->is_cyclic
    # ### $graph
    ### $sptg
    my @successors = Graph_SPT_successors($sptg,$diameter_start);
    ### @successors
    my @neighbours = $sptg->neighbours($diameter_start);
    ### @neighbours

    Graph_SPT_counts($graph,$sptg);
    print "k=$k\n";
    print "  vertices ",join(' ',$graph->vertices),"\n";
    print "  diameter $diameter from $diameter_start to $diameter_end\n";
    my $diameter_count = $sptg->get_vertex_attribute($diameter_end,'count');
    print "  count $diameter_count\n";
    if ($verbose && $graph->vertices < 32) {
      print "  [$diameter_num_vertices_by_path]  ",join(' ',@diameter_path),"\n";
      foreach my $v (sort $graph->vertices) {
        my $count = $sptg->get_vertex_attribute($v,'count');
        my $p = $sptg->get_vertex_attribute($v,'p');
        print "  $v  $count  p=$v\n";
      }
    }
    push @values, $diameter_count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;

  sub Graph_SPT_counts {
    my ($graph,$sptg) = @_;
    my $start = $sptg->get_graph_attribute('SPT_Dijkstra_root');
    $sptg ->set_vertex_attribute ($start,'count',1);
    foreach my $from (sort {($sptg->get_vertex_attribute($a,'weight') || 0)
                              <=>
                              ($sptg->get_vertex_attribute($b,'weight') || 0)}
                      $sptg->vertices) {
      my $target_distance
        = ($sptg->get_vertex_attribute($from,'weight') || 0) + 1;
      my $from_count = $sptg ->get_vertex_attribute($from,'count');
      ### from: $from . ' weight ' .($sptg->get_vertex_attribute($from,'weight') || 0)
      ### $from_count
      ### $target_distance
      foreach my $to ($graph->neighbours($from)) {
        if (($sptg->get_vertex_attribute($to,'weight') || 0)
            == $target_distance) {
          ### to: $to . ' weight ' .($sptg->get_vertex_attribute($to,'weight') || 0)
          $sptg ->set_vertex_attribute
            ($to,'count',
             $from_count + ($sptg ->get_vertex_attribute($to,'count') || 0));
        } else {
          ### skip: $to . ' weight ' .($sptg->get_vertex_attribute($to,'weight') || 0)
        }
      }
    }
  }
  sub Graph_SPT_successors {
    my ($sptg, $v) = @_;
    return grep { my $p = $sptg->get_vertex_attribute($_,'p');
                  defined $p && $p eq $v}
      $sptg->neighbours($v);
  }

}
{
  # twinalternate graph view
  require MyGraphs;
  foreach my $k (0 .. 5) {
    print "k=$k\n";
    my $graph = twinalternate_Graph($k);
    # MyGraphs::Graph_view($graph);
    MyGraphs::Graph_xy_print($graph);
 }
  exit 0;
}

{
  # TA domnum sans connections

  require MyGraphs;
  require Graph::Maker::TwinAlternateAreaTree;
  foreach my $k (0 .. 10) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   # vertex_name_type => 'xy',
                                   undirected => 1);
    my $whole_domnum = MyGraphs::Graph_tree_domnum($graph);
    # MyGraphs::Graph_view($graph);

    my $aa = TA_a($k);
    my $cc = TA_c($k);
    my $ee = TA_e($k);
    my @dels = (0,$aa,$cc,$ee);
    my $dels = join(',',@dels);
    foreach my $elem (['s',0],
                      ['aa',$aa],
                      ['cc',$cc],
                      ['ee',$ee],
                     ) {
      my ($name, $v) = @$elem;
      if ($k >= 2) {
        $graph->has_vertex($v) or die "k=$k no vertex $name = $v";
      }
      $graph->delete_vertex($v);
    }

    my @components = $graph->connected_components;
    my $num_components = scalar @components;
    my @sizes = map {scalar @$_} @components;
    @sizes = sort {$a<=>$b} @sizes;
    my $sizes = join(',',@sizes);

    my $domnum = MyGraphs::Graph_tree_domnum($graph);
    print "k=$k  delete $dels\n";
    print "  components $num_components  $sizes  domnum $domnum whole $whole_domnum\n";
    # MyGraphs::Graph_view($graph);
  }
  exit 0;
}
{
  # TA domnum by components

  # 0, 0, 1, 3, 5, 6, 15, 26, 44, 92, 194

  require MyGraphs;
  require Graph::Maker::TwinAlternateAreaTree;
  foreach my $k (0 .. 12) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   vertex_name_type => 'xy',
                                   undirected => 1);
    if ($k == 8) {
      MyGraphs::Graph_view($graph);
    }
    my @deletes;
    my %dominated;
    foreach my $v ($graph->vertices) {
      next unless $graph->has_vertex($v);
      next unless $graph->vertex_degree($v) <= 1;
      push @deletes, $v, $graph->neighbours($v);
      foreach my $d ($graph->neighbours($v)) {
        $dominated{$d} = 1;
      }
    }
    $graph->delete_vertices(@deletes);
    $graph->delete_vertices(map {$graph->has_vertex($_)
                                   && $graph->vertex_degree($_) <= 1
                                   ? ($_) : ()}
                            keys %dominated);

    my @components = $graph->connected_components;
    my $num_components = scalar @components;
    my @sizes = map {scalar @$_} @components;
    @sizes = sort {$a<=>$b} @sizes;
    my $sizes = join(',',@sizes);
    print "k=$k  $num_components  $sizes\n";

    if ($k == 8) {
      MyGraphs::Graph_view($graph);
    }
  }
  exit 0;
}
{
  # Spred or DpredFirst

  my $f = DpredFirstInf_flat()->union(SpredInf_flat());
  $f = $f->MyFLAT::reverse;
  $f = $f->as_dfa;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::separate_sinks;
  $f->MyFLAT::view;

  $f = SpredInf_flat();
  $f = $f->MyFLAT::reverse;
  $f = $f->as_dfa;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f = $f->MyFLAT::separate_sinks;
  $f->MyFLAT::view;
  exit 0;
}


{
  # alternate graph Wiener index

  #  8
  #
  #  7                               *---*
  #                                  |   |
  #  6                           *---*---*
  #                              |   |
  #  5                       *---*---*---*
  #                          |   |   |   |
  #  4                   *---*---*---*---*
  #                      |   |   |   |
  #  3               *---*---*---*---*---*
  #                  |   |   |   |   |   |
  #  2           *---*---*---*---*---*---*
  #              |   |   |   |   |   |
  #  1       *---*---*---*---*---*---*---*
  #          |   |   |   |   |   |   |   |
  #  0   *---*   *---*   *---*   *---*   *
  #
  #      0   1   2   3   4   5   6   7   8
  #
  #
  #  4                   *
  #                      |
  #  3               *---*---*
  #                  |   |   |
  #  2           *---*---*---*---*
  #              |   |   |   |   |
  #  1       *---*---*---*---*---*---*
  #          |   |   |   |   |   |   |
  #  0   *---*   *---*   *---*   *---*
  #
  #      0   1   2   3   4   5   6   7   8

  # Manhattans
  # W1(n,x,y) = sum(x2=x,n, sum(y2=y,min(x2,n-1), \
  #                 my(d=abs(x-x2)+abs(y-y2)); \
  #                 print(x","y" to "x2","y2" = ",d); d; \
  #                 d));
  # W(1)
  # W(2) + E(2) + E(1)
  # vector(5,k, W(2^k)+E(2^k)+E(2^k-1))
  # E(2)
  # E(1)
  # 1+2+3+4 + 1+2+3 + 1+2 + 1 == 20
  # 1+2+3+2 + 1+2+1 + 1+0 + 1 == 14
  #       *       *
  # 1+2+2+3+1+1+2+1+1

  # vector(6,k,k--; E(2^k))
  # vector(6,k,k--; E(2^k-1))

  require Graph::Maker::PlanePath;
  require MyGraphs;
  my @values;
  foreach my $k (0 .. 9) {
    require Graph::Maker::PlanePath;
    my $graph = Graph::Maker->new('planepath',
                                  undirected=>1,
                                  level=>$k,
                                  planepath => 'AlternatePaper',
                                  vertex_names_type => 'xy',
                                 );
    MyGraphs::Graph_xy_print($graph);

    my $apsp = $graph->APSP_Floyd_Warshall;
    # my $diameter = $apsp->diameter;
    my @vertices = $graph->vertices;
    my $W = MyGraphs::Graph_Wiener_index($graph);

    print "$k  W=$W\n";
    push @values, $W;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  #
  require MyFLAT;
  my @dir4_state_table = ([0,1],  # 0
                          [0,3],  # 1
                          [2,3],  # 2
                          [2,1],  # 3
                         );
  foreach my $dir (0..3) {
    my $f = MyFLAT::aref_to_FLAT_DFA
      (\@dir4_state_table,
       name      => "dir=$dir",
       starting  => 0,
       accepting => $dir);
    $f = $f->MyFLAT::reverse;
    $f = $f->MyFLAT::minimize(verbose=>1);
    $f->MyFLAT::view;
  }
  exit 0;
}
{
  require MyGraphs;
  MyGraphs::Graph_view(MyAlternate::parallelogram_Graph(2,5));
  MyGraphs::Graph_view(MyAlternate::parallelogram_Graph(4,1));
  exit 0;
}

{
  my $f = GRS_flat(1);
  my @f = ($f);
  foreach my $i (1 .. 3) {
    $f[$i] = $f[$i-1]->MyFLAT::digits_increment->MyFLAT::minimize(verbose=>1);
  }
  $f = $f->intersect(@f);
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::digits_increment(add => -3)->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # 4,16;  # 0
  # 5,17;  # 1
  # 6,18;  # 2
  # 7,19;  # 3
  # 8,22;  # 4
  # 9,20;  # 5

  # 10,21;  # 6
  # 11,22;  # 7
  # 4,17;  # 8
  # 5,18;  # 9
  # 6,16;  # 10
  # 7,17;  # 11

  # 4,16;  # 12
  # 5,17;  # 13
  # 6,18;  # 14
  # 7,19;  # 15
  # 10,20;  # 16
  # 8,21;  # 17
  # 9,22;  # 18
  # 10,23;  # 19
  # 5,16;  # 20
  # 6,17;  # 21
  # 4,18;  # 22
  # 5,19;  # 23

  my @state_table;
  my $dpb_to_state = sub {
    my ($dir,$pos,$prev_bit) = @_;
    return $dir + 4*($pos + 2*$prev_bit);
  };
  my $state_to_dpb = sub {
    my ($state) = @_;
    my $dir = $state % 4; $state = int($state/4);
    my $pos = $state % 2; $state = int($state/2);
    return ($dir,$pos,$state);
  };
  my $state_to_str = sub {
    my ($state) = @_;
    my ($dir,$pos,$prev_bit) = $state_to_dpb->($state);
    return "dir=$dir,pos=$pos,prev=$prev_bit";
  };
  foreach my $dir (0 .. 3) {
    foreach my $pos (0, 1) {
      foreach my $prev_bit (0, 1, 2) {
        next if $prev_bit==2 && $pos==1;

        my $state = $dpb_to_state->($dir,$pos,$prev_bit);
        foreach my $bit (0,1) {
          my $new_dir = $dir;
          if ($prev_bit!=2 && $bit != $prev_bit) {
            $new_dir += ($pos % 2 ? 1 : -1);
            $new_dir %= 4;
          }
          my $new_pos = 1 - $pos;
          my $new_state = $dpb_to_state->($new_dir,$new_pos,$bit);
          $state_table[$state]->[$bit] = $new_state;
        }
      }
    }
  }

  my @state_accepting;
  my @dir_accepting;
  foreach my $state (0 .. $#state_table) {
    my ($dir,$pos,$prev_bit) = $state_to_dpb->($state);
    if ($prev_bit == 2) {
      push @{$dir_accepting[$dir]}, $state;
      $state_accepting[$state] .= "i$dir,";
    } else {
      my $new_state = $state_table[$state]->[0];
      my ($new_dir,$new_pos,$bit) = $state_to_dpb->($new_state);
      $bit==0 or die;
      push @{$dir_accepting[$new_dir]}, $state;
      $state_accepting[$state] .= "$new_dir";
    }
  }

  foreach my $state (0 .. $#state_table) {
    print "  $state_table[$state]->[0],$state_table[$state]->[1];  # $state = ",
      $state_to_str->($state),"      ",$state_accepting[$state]//'', "\n";
  }

  my $dir_mod4_by_states = sub {
    my ($n) = @_;
    my $state = 0;
    foreach my $bit (bit_split_lowtohigh($n), 0) {
      $state = $state_table[$state]->[$bit];
    }
    return $state % 4;
  };

  foreach my $n (0 .. 16) {
    my $d = dir($n) % 4;
    my $s = $dir_mod4_by_states->($n);
    print "$n $d $s\n";
  }

  my $starting = $dpb_to_state->(0,0,2);
  {
    push @state_table, [0,0]; my $new_starting = $#state_table;
    push @state_table, [0,0]; my $non = $#state_table;
    foreach my $bit (0,1) {
      push @state_table, [0,0]; my $new_state = $#state_table;
      $state_table[$new_starting]->[$bit] = $new_state;
      $state_table[$new_state]->[$bit] = $state_table[$starting]->[$bit];
      $state_table[$new_state]->[1-$bit] = $non;
      $state_table[$new_starting]->[$bit] = $new_state;
      $state_table[$non]->[$bit] = $non;
    }
    $starting = $new_starting;
  }

  my $HtoL = 0;
  my $radix = 4;
  require MyFLAT;
  foreach my $dir (0..3) {
    my $f = MyFLAT::aref_to_FLAT_DFA
      (\@state_table,
       name           => "dir=$dir",
       starting       => $starting,
       accepting_list => $dir_accepting[$dir]);
    if ($HtoL) {
      $f = $f->MyFLAT::reverse;
    }
    if ($radix == 4) {
      $f = $f->MyFLAT::binary_to_base4
        (direction => ($HtoL ? "hightolow" : "lowtohigh"));
    }
    # $f = $f->MyFLAT::minimize(verbose=>1);
    $f->MyFLAT::view;

    # foreach my $n (0 .. 256) {
    #   foreach my $zeros ([], [0], [0,0], [0,0,0]) {
    #     my @digits = (digit_split_lowtohigh($n,$radix), @$zeros);
    #     my $d = (dir($n) % 4 == $dir ? 1 : 0);
    #     my $str = join('',@digits);
    #     if ($HtoL) { $str = reverse $str; }
    #     my $s = ($f->contains($str) ? 1 : 0);
    #     unless ($d == $s) {
    #       print "dir=$dir n=$n [$str]  $d $s\n";
    #     }
    #   }
    # }
  }
  exit 0;
}

{
  # terminal Wiener index
  # 1, 2, 4, 4, 6, 8, 10, 16, 18
  # = TAdiameter since only start,end are degree-1

  require MyGraphs;
  foreach my $k (0 .. 8) {
    my $graph = alternate_Graph($k);
    my $TW = MyGraphs::Graph_terminal_Wiener_index($graph);
    print "k=$k  $TW\n";
  }
  exit 0;
}
{
  # TA tree halves
  # not in OEIS: 1,2,3,3,4,6,6,5,6,8,9,9,11,13,11,9,10,12,12,12,16,20,19,16,17,20,21,22,25,25,20
  # not in OEIS: 1,2,2,4,5,4,4,6,8,9,8,8,10,11,11,10,10,12,13,14,17,19,18,16,15,18,22,22,21,22,23

  # WidthS inf
  # 1,2,3,3,4,6,6,5,6,8,9,9,11,13,11,9,10,12,12,12,16,20,19,16,17,20,21,22,25,25,20

  require MyGraphs;
  require Graph::Maker::TwinAlternateAreaTree;
  my @values;
  foreach my $k (2 .. 12) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   vertex_name_type => 'xy',
                                   undirected => 1);
    # MyGraphs::Graph_view($graph);
    # $graph->delete_edge ($graph);
    foreach my $root ('0,0',
                      # '-1,0',
                     ) {
      $graph->has_vertex($root) or die;
      my @rows = MyGraphs::Graph_vertices_by_depth
        ($graph, root => $root, cmp => \&MyGraphs::cmp_alphabetic);
      @rows = map {scalar(@$_)} @rows;
      print join(',',@rows),"\n";
    }
  }
  exit 0;
}
{
  # Hamiltonian
  require MyGraphs;
  foreach my $k (0 .. 8) {
    my $graph = alternate_Graph($k);
    print "k=$k\n";
    {
      my $c = MyGraphs::Graph_is_Hamiltonian($graph) ? 1 : 0;
      my $p = MyGraphs::Graph_is_Hamiltonian($graph, type => 'path') ? 1 : 0;
      print "  Hamiltonian cycle $c path $p\n";
    }
    MyGraphs::Graph_delete_hanging_cycles($graph);
    {
      my $c = MyGraphs::Graph_is_Hamiltonian($graph) ? 1 : 0;
      my $p = MyGraphs::Graph_is_Hamiltonian($graph, type => 'path') ? 1 : 0;
      print "  sans hanging cycle $c path $p\n";
    }
  }
  exit 0;
}

{
  # Cheeger constant   separating edges / smaller set

  require Math::BigRat;
  my $graph = alternate_Graph(4);
  my $num_vertices = $graph->vertices;
  my $num_edges = $graph->edges;
  my $max_half_vertices = int($num_vertices/2);

  # my $minimum = Math::BigRat->new()

  exit 0;
}
{
  # TAparentDir flat reversal

  # TAparentDir1
  #    TAparentDir0 011111

  require MyFLAT;
  # TAparentDir1_flat()->MyFLAT::view;
  my $f = TAparentDir1_flat()->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  MyFLAT::FLAT_print_tikz($f);

  $f = $f->MyFLAT::skip_initial;
  $f->MyFLAT::binary_to_base4(direction=>'lowtohigh')->MyFLAT::view;

  exit 0;
}

{
  # TAparentDir flat reversal

  # TAparentDir2
  #   odd even even odd
  # 1 000 1111 0000 111
  #       \-------/

  require MyFLAT;
  my $f = TAparentDir2_flat();
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  MyFLAT::FLAT_print_tikz($f);

  # no
  # my $flip = TAparentDir0_flat()
  #   ->MyFLAT::FLAT_rename(symbols_map => {0=>1,1=>0});
  # MyFLAT::FLAT_check_is_equal(TAparentDir2_flat(), $flip);

  exit 0;
}

{
  # TAparentDir flat reversal

  # TAparentDir0
  #   odd even even odd
  # 0 111 0000 1111 000
  #       \-------/
  #                10 at even pos
  #  01 at odd pos

  require MyFLAT;
  # TAparentDir0_flat()->MyFLAT::view;
  my $f = TAparentDir0_flat()->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  MyFLAT::FLAT_print_tikz($f);

  $f = $f->MyFLAT::skip_initial;
  $f->MyFLAT::binary_to_base4(direction=>'lowtohigh')->MyFLAT::view;

  exit 0;
}




{
  # TAparentDir FSM state reversal

  require MyFSM;
  my %table;
  foreach my $state (0 .. $#MyAlternate::TAparentDir_state_table) {
    foreach my $bit (0,1) {
      $table{$state}->{$bit}
        = $MyAlternate::TAparentDir_state_table[$state]->[$bit];
    }
  }
  my %accepting = (0 => 'zero',
                   1 => 'one',
                   # 2 => 'two',
                   # 3 => 'three',
                  );
  my $fsm = MyFSM->new(table => \%table,
                       initial => 3,
                       accepting => \%accepting,
                       title => 'TAparentDir',
                       flow => 'east',
                      );
  # $fsm->simplify (verbose => 1);

  $fsm = $fsm->reverse;

  # $fsm->renumber_states;
  $fsm->simplify (verbose => 1);
  # $fsm->renumber_styates;

  $fsm->view;
  $fsm->show;
  exit 0;
}
{
  # twin alternate widths

  require Graph::Maker::TwinAlternateAreaTree;
  my @values;
  foreach my $k (12) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   # vertex_name_type => 'xy',
                                   undirected => 1);
    my %seen;
    my $root = '0';
    $seen{$root} = 1;
    my @pending = ($root);
    my $depth = 0;
    while (@pending) {
      my $width = scalar(@pending);
      push @values, $width;
      print "depth $depth  width $width  ",join(',',@pending),"\n";
      @pending = grep {!$seen{$_}++} map {$graph->neighbors($_)} @pending;
      last if $depth > 2**($k/2-2);
    }
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # twin alternate area tree diameter

  # TAdiameter(k) = vector_modulo([7, 10],k) * 2^floor(k/2) - 2*k - 7
  # 0,1,3,7,13,23,37,59,89
  # A053599 Number of nonempty subsequences {s(k)} of 1...n such that the difference sequence is palindromic.

  # k=6 diameter 37
  #   at -7,7 to 7,0
  #   at 7,0 to -7,7
  # k=7 diameter 59
  #   at -7,7 to 15,0
  #   at 15,0 to -7,7
  
  # attained at furthest corners of the rhombus
  # *---*
  #     |
  #     *---*
  #


  require Graph::Maker::TwinAlternateAreaTree;
  my @values;
  foreach my $k (0 .. 8) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   vertex_name_type => 'xy',
                                   undirected => 1);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter || 0;

    print "k=$k diameter $diameter\n";
    $graph->for_shortest_paths(sub {
                                 my ($t, $u,$v, $n) = @_;
                                 my $len = $t->path_length($u,$v);
                                 if ($len == $diameter) {
                                   print "  at $u to $v\n";
                                 }
                               });
    push @values, $diameter;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # twin alternate area tree corner eccentricity

  # corner k=4 at 2,1
  # corner k=5 at 0,3
  # corner k=6 at 4,3
  # corner k=7 at 0,7
  #
  # eccentricity 0,1,3,5,11,14,29,36,67

  require Graph::Maker::TwinAlternateAreaTree;
  my @values;
  foreach my $k (2 .. 8) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   vertex_name_type => 'xy',
                                   undirected => 1);
    # require MyGraphs;
    # MyGraphs::Graph_view($graph);
    my $half = 2**int(($k-1)/2);
    my $corner = ($k&1 ? '0,'.($half-1) : "$half,".($half-1));
    ### $half
    ### $corner
    if (! $graph->has_vertex($corner)) {
      print "corner $corner  half $half\n";
      require MyGraphs;
      MyGraphs::Graph_view($graph);
      sleep 100;
    }
    my $degree = $graph->vertex_degree($corner);
    my $eccentricity = $graph->vertex_eccentricity($corner);
    if ($eccentricity > 10e10) { $eccentricity = 0; }  # Inf when one vertex

    print "k=$k corner $corner deg $degree, eccentricity $eccentricity \n";
    $graph->for_shortest_paths(sub {
                                 my ($t, $u,$v, $n) = @_;
                                 if ($u eq $corner) {
                                   my $len = $t->path_length($u,$v);
                                   if ($len == $eccentricity) {
                                     print "  at $u to $v\n";
                                   }
                                 }
                               });
    push @values, $eccentricity;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # twin alternate area tree root eccentricity

  # attained at furthest sharp corner of the rhombus
  # A077866 Expansion of (1-x)^(-1)/(1-x-2*x^2+2*x^3).


  require Graph::Maker::TwinAlternateAreaTree;
  my @values;
  foreach my $k (0 .. 8) {
    my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                                   level=>$k,
                                   vertex_name_type => 'xy',
                                   undirected => 1);
    my $root = '0,0';
    my $eccentricity = $graph->vertex_eccentricity($root);
    if ($eccentricity > 10e10) { $eccentricity = 0; }  # Inf when one vertex

    print "k=$k eccentricity $eccentricity\n";
    $graph->for_shortest_paths(sub {
                                 my ($t, $u,$v, $n) = @_;
                                 if ($u eq $root) {
                                   my $len = $t->path_length($u,$v);
                                   if ($len == $eccentricity) {
                                     print "  at $u to $v\n";
                                   }
                                 }
                               });
    push @values, $eccentricity;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # HOG graphs
  # AlternatePaper k=3..5 not

  require MyGraphs;
  my @graphs;
  foreach my $k (0 .. 5) {
    my $graph = alternate_Graph($k);
    my $num_vertices = $graph->vertices;
    my $num_edges = $graph->edges;
    my $diameter = $graph->diameter;
    print "$num_vertices vertices $num_edges edges, diameter $diameter\n";
    push @graphs, $graph;
  }
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}





{
  # x=2 010  mask 0011
  # x=6 110  mask 0011  want 0001
  #
  # 7 = 0111
  # 6 = 0110   <--
  # 5 = 0101
  # 4 = 0100   0
  # 3 = 0011
  # 2 = 0010   <--
  # 1 = 0001
  # 0 = 0000   0

  my $x = 2;
  my $lower = sub {
    my ($x,$y) = @_;
    # $x ^= $x>>1;
    my $mask = ($x ^ ($x-1));
    # print " $mask\n";
    # $mask ^= ($mask+1)<<1;
    return ($y & $mask ? 1 : 0);
  };
  foreach my $y (0 .. 63) {
    print $lower->($x,$y) ? '1' : '-';
  }
  print "\n";
  exit 0;
}


{
  # (10)* 00 01x
  # 1 (01)* 11 0x
  #
  require MyFLAT;
  require FLAT::Regex;
  my $f = FLAT::Regex->new ('(00|10)* | (10|11)*')->as_dfa
    ->MyFLAT::prefix_accepting
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;
  exit 0;
}
{
  # (10)* 00 01x
  # 1 (01)* 11 0x
  #
  my $f = fGpred_flat()->union(fVpred_flat())->as_dfa;
  $f = $f->MyFLAT::prefix_accepting;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;
}
{
  # (10)* 00 01x
  # 1 (01)* 11 0x
  #
  fNonGpred_flat()->union(fNonVpred_flat())
    # ->as_dfa
    # ->MyFLAT::reverse
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::eventually_accepting
    ->MyFLAT::view;
  exit 0;
}
{
  # fB by states

  # [0] x = 0 d            [3] @x = 1 @d       
  # [1] v = 1 @d           [4] @v = 0 d            
  # [2] d = 0 x | 1 @v     [5] @d = 1 @x | 0 v 
  # [6] = 0*   [7] = 1*
  # [8],[9] = (10)*

  require MyFLAT;
  require FLAT::DFA;
  my $f = FLAT::DFA->new;
  my @states = $f->add_states(11);
  $f->add_transition(0,2, 0);    $f->add_transition(3,6, 0);
  $f->add_transition(0,7, 1);    $f->add_transition(3,5, 1);
  $f->add_transition(1,8, 0);    $f->add_transition(4,2, 0);
  $f->add_transition(1,5, 1);    $f->add_transition(4,9, 1);
  $f->add_transition(2,0, 0);    $f->add_transition(5,3, 1);
  $f->add_transition(2,4, 1);    $f->add_transition(5,1, 0);

  $f->add_transition(6,6, 0);
  $f->add_transition(7,7, 1);
  $f->add_transition(8,9, 1);
  $f->add_transition(9,8, 0);
  $f->set_accepting(0 .. 10);

  $f->add_transition(10,8, 0);
  $f->add_transition(10,7, 1);

  {
    my $f = $f->clone;
    $f->set_starting(0);
    $f = $f->MyFLAT::minimize(verbose=>1);
    $f->MyFLAT::view;
    MyFLAT::FLAT_show_breadth($f, 4);
    # $f = $f->MyFLAT::prefix_accepting;

    fBpred_hullsaround_init();
    my $fr = $fRpred_hullsaround_flat;
    print $fr->contains('11111111111') ? "ones\n" : "not ones\n";
    print $fr->contains('010101010') ? "third\n" : "not third\n";

    $fr = $fr->MyFLAT::minimize(verbose=>1);
    $fr->MyFLAT::view;
    # $fr = $fr->MyFLAT::prefix_accepting;
    MyFLAT::FLAT_show_breadth($fr, 4);

    MyFLAT::FLAT_check_is_equal($f,$fr);
  }
  {
    my $f = $f->clone->MyFLAT::as_nfa;
    $f->set_starting(1,2);
    $f = $f->as_dfa;
    # $f = $f->MyFLAT::prefix_accepting;

    fBpred_hullsaround_init();
    my $fl = $fLpred_hullsaround_flat;
    $fl = $fl->MyFLAT::prefix_accepting;

    MyFLAT::FLAT_check_is_equal($f,$fl);
  }
  exit 0;
}

{
  # location where first have s = x+y

  my $path = Math::PlanePath::AlternatePaper->new;
  my $target = 0;
  my $prev_n = 0;
  for (my $n = 0; $n < 20000; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my $s = $x+$y;
    if ($s == $target) {
      my $n2 = cnv($n,10,2);
      my $n_diff = $n - $prev_n;
      my $n_diff2 = cnv($n_diff,10,2);
      print "s=$s at N=$n [$n2]  $x,$y  $n_diff [$n_diff2]\n";
      $target++;
      $prev_n = $n;
    }
  }
  exit 0;
}


{
  $hullsaround_segs_outer->print;
  $hullsaround_segs_inner->print;
  exit 0; 
}

{
  # conf expansions
  my $c = MyAlternateConf->initial->expand(1)->expand(0)->expand(0);
  $c->print;
  $c->expand(0)->print;
  $c->expand(1)->print;
  exit 0;
}
{
  # fR

  fBpred_hullsaround_init();
  my $fl = $fLpred_hullsaround_flat;
  $fl = $fl->MyFLAT::prefix_accepting;

  my $smaller = FLAT::Regex->new ('(10)* 0 (01)*')->as_dfa
    ->MyFLAT::prefix_accepting;
  $fl = $fl->intersect($smaller);
  $fl = $fl->MyFLAT::prefix_accepting;
  $fl->{'name'} = 'diagonal';

  # base-4 2s rationals ending ...000 or ...1111
  # base-4 2s odd + 5/6
  # +2/3 exactly
  #
  my $twos = FLAT::Regex->new ('(00 | 10)*')->as_dfa;
  my $twos_ending_two = FLAT::Regex->new ('(00 | 10)* 10')->as_dfa;
  my $f = $twos->MyFLAT::concat(FLAT::Regex->new('0*')->as_dfa)
    ->union
    (
     # $twos->MyFLAT::prefix_accepting
     # ->as_dfa->MyFLAT::fraction_also_nines,

     FLAT::Regex->new ('(00 | 10)* 01 1*')->as_dfa,

     # $twos_ending_two->MyFLAT::concat(MyFLAT::fraction_digits(5,6, radix=>2)),
     
     MyFLAT::fraction_digits(5,6, radix => 2)
    )
    ->as_dfa;
  $f = $twos;
  $f = $f->MyFLAT::prefix_accepting;

  # MyFLAT::FLAT_show_breadth($fl, 8);
  MyFLAT::FLAT_check_is_equal($f,$fl);
  exit 0;

  # GP-Test  alternate_fpoint(1/2^7) == 1/16*(1+I)
}

{
  # fR

  fBpred_hullsaround_init();
  my $fr = $fRpred_hullsaround_flat;
  $fr = $fr->MyFLAT::prefix_accepting;
  # $fr = $fr->MyFLAT::eventually_accepting;
  # MyFLAT::FLAT_show_breadth($fr, 6);
  # $fr = $fr->MyFLAT::minimize(verbose=>1);

  my $f = FLAT::Regex->new
    ('  (00 | 01)* 0*
      | (00 | 01)* 01 (10)*
      | (00 | 01)* 001*
      | (10)*
     ')->as_dfa;
  $f = $f->MyFLAT::prefix_accepting;
  # $f = $f->MyFLAT::eventually_accepting;
  $f->MyFLAT::set_name('digits');
  # MyFLAT::FLAT_show_breadth($f, 6);

  MyFLAT::FLAT_check_is_equal($f,$fr);

  # base-4 1s rationals ending ...000 or ...1111
  # base-4 1s odd + 2/3
  # +2/3 exactly
  #
  my $binary_odd_flat = FLAT::Regex->new ('(0|1)* 1')->as_dfa;
  my $ones = FLAT::Regex->new ('(00 | 01)*')->as_dfa;
  my $odds = $ones->intersect($binary_odd_flat)->as_dfa;
  $ones = $ones->MyFLAT::prefix_accepting;
  $f = $ones->MyFLAT::concat(FLAT::Regex->new('0*')->as_dfa)
    ->union($ones->as_dfa->MyFLAT::fraction_also_nines->as_dfa,
            $odds->MyFLAT::concat(MyFLAT::fraction_digits(2,3, radix => 2)),
            MyFLAT::fraction_digits(2,3, radix => 2))
    ->as_dfa;
  $f = $f->MyFLAT::prefix_accepting;

  MyFLAT::FLAT_check_is_equal($f,$fr);

  # fracper_to_frac([ [1,0]],2) == 2/3
  # alternate_fpoint(1/3) == 1
  # alternate_fpoint(6/64 + 2/3/64) == 1/4
  # alternate_fpoint(1/64) == 1/8
  # to_binary(floor((1+ 6/64 + 2/3/64)*2^32))
  # to_binary(floor((1+ 1/64 + 2/3/64)*2^32))
  # alternate_fpoint(1/64 + 2/3/64) == 1/8
  # alternate_fpoint(1/64 + 1/3/64) == 1/4
  #                                              . _ _ _  
  # to_binary(floor((1+ 1/64 + 2/3/64)*2^16)) == 10000011010101010
  # to_binary(floor((1+ 1/64 + 1/3/64)*2^16)) == 10000010101010101
  # to_binary(floor((1+ 1/64         )*2^16)) == 10000010000000000

  #       *--->*
  #       |    ^
  #       v    |
  #  *--->*    *
  exit 0;
}

{
  # fB

  fBpred_hullsaround_init();
  my $fb = $fBpred_hullsaround_flat;
  $fb = $fb->MyFLAT::prefix_accepting;
  # $fb = $fb->MyFLAT::eventually_accepting;
  MyFLAT::FLAT_show_breadth($fb, 6);
  # $fb = $fb->MyFLAT::minimize(verbose=>1);

  my $f = FLAT::Regex->new
    ('  (00 | 01)* 0*
      | (00 | 01)* 01 (10)*
      | (00 | 10)* (0* | 01*)
      | (10 | 11)* (0* | 01* | 1*)
     ')->as_dfa;
  $f = $f->MyFLAT::prefix_accepting;
  # $f = $f->MyFLAT::eventually_accepting;
  $f->MyFLAT::set_name('digits');
  MyFLAT::FLAT_show_breadth($f, 6);

  MyFLAT::FLAT_check_is_equal($f,$fb);

  exit 0;
}

{
  # fNonB

  fBpred_hullsaround_init();
  my $f = $fNonBpred_hullsaround_flat;
  # $f = $f->MyFLAT::prefix_accepting;
  $f = $f->MyFLAT::eventually_accepting;
  MyFLAT::FLAT_show_breadth($f, 6);

  # $f = $f->MyFLAT::minimize(verbose=>1);

  $f = FLAT::Regex->new
    ('  (00 | 01)* (0* | 001*)
      | (00 | 10)* (0* | 01*)
      | (10 | 11)* (0* | 01* | 1*)
     ')->as_dfa;
  $f = $f->complement;
  # $f = $f->MyFLAT::prefix_accepting;
  $f = $f->MyFLAT::eventually_accepting;
  MyFLAT::FLAT_show_breadth($f, 6);

  # alternate_fpoint(12/32) == 3/4 + 1/4*I
  # alternate_fpoint(6/64) == 3/8 + 1/8*I
  # dir(6) == 2
  # alternate_fpoint(6/64 + 2/3/64) == 1/4
  exit 0;
}

{
  # hulls around

  use constant::defer Vpred_flat => sub {
    Xpred_flat()->MyFLAT::FLAT_rename(symbols_map => {0=>1,1=>0})
      ->MyFLAT::set_name('Vpred')
      ->MyFLAT::minimize;
  };

  # Xpred_flat()->MyFLAT::view;
  # Vpred_flat()->MyFLAT::view;

  my $f = Vpred_flat()->union(Xpred_flat(),Gpred_flat())
    ->MyFLAT::prefix_accepting
    ->as_dfa
    ->MyFLAT::minimize;
  # $f->MyFLAT::view;
  MyFLAT::FLAT_show_breadth($f, 4);

  fBpred_hullsaround_init();
  my $fb = $fBpred_hullsaround_flat
    ->MyFLAT::prefix_accepting
    ->MyFLAT::minimize(verbose=>1)
    ;
  $fb->MyFLAT::view;

  MyFLAT::FLAT_show_breadth($fb, 4);

  MyFLAT::FLAT_check_is_equal($f,$fb);

  # alternate_point(14) == 3 + I
  exit 0;
}
{
  # DFA views
  # TurnLpred_flat()->MyFLAT::view;
  # TurnRpred_flat()->MyFLAT::view;
  TurnLpred_flat()->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  TurnRpred_flat()->MyFLAT::reverse->MyFLAT::minimize->MyFLAT::view;
  exit 0;
}

{
  # alternate graph paths attaining diameter

  my @values;
  foreach my $k (0 .. 10) {
    my $graph = alternate_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter      = $apsp->diameter;

    print "k=$k diameter $diameter\n";
    $graph->for_shortest_paths(sub {
                                 my ($t, $u,$v, $n) = @_;
                                 my $len = $t->path_length($u,$v);
                                 if ($len == $diameter) {
                                   print "  at $u to $v\n";
                                 }
                               });
  }
  exit 0;
}


{
  # alternate graph
  # diameter is PathLen, start to end 1,2,4,8,12,18,26,36,52,70
  #
  # centre 0+0 1+0   # 0
  #        1+0       # 1
  #        1+1       # 2
  #        0+2       # 3
  #        -1+1      # 4
  #        -3+0      # 5
  #        -2-3      # 6
  #        0-6       # 7
  #        7-5 6-4   # 8
  #        11+0      # 9


  my $graph = alternate_Graph(3);
  # graph_show($graph);
  print "num edges    ",scalar($graph->edges),"\n";
  print "num vertices ",scalar($graph->vertices),"\n";
  print "diameter     ",scalar($graph->diameter),"\n";

  my @values;
  foreach my $k (0 .. 10) {
    my $graph = alternate_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter      = $apsp->diameter;
    my $radius        = $apsp->diameter;
    my $shortest_path = $apsp->shortest_path;
    my $average_path  = $apsp->average_path_length;
    my $longest_path  = $apsp->longest_path;
    my @longest_path  = $apsp->longest_path;
    my @centre        = $apsp->center_vertices;
    print "k=$k  diameter=$diameter radius=$radius shortest=$shortest_path longest=$longest_path avg=$average_path\n";
    print "  centre  ",join(' ',@centre),"\n";
    print "  longest ",join(' ',@longest_path),"\n";
    push @values, $diameter;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

}
{
  # other n state transitions

  require MyFSM;
  my %table;
  my %accepting = (same => 1);
  my @pending = ([1,-1, 0,0, 0], [1,-1, 0,0, 1]);
  my $state_to_name = sub {
    my ($state) = @_;
    my ($old_s,$new_s, $delta_x,$delta_y, $bit) = @$state;
    # if ($old_s == $new_s && $delta_x == 0 && $delta_y == 0) {
    #   return "same";
    # }
    my $out = ($bit ^ ($delta_x+$delta_y)) & 1;
    my $flip = ($bit == $out ? "same" : "flip");
    return "$old_s,$new_s $delta_x,$delta_y $flip$bit$out";
    return "$old_s,$new_s $delta_x,$delta_y ${bit}out$out";
  };
  my $initial = $state_to_name->($pending[0]);
  my %seen;
  my %allow_delta = ('0,0' => 1,
                     '1,0' => 1,
                     '-1,0' => 1,
                     '1,1' => 1,
                     '-1,-1' => 1,
                    );
  my $b = Math::Complex->new(1,1);
  while (@pending) {
    my $state = shift @pending;
    my ($old_s,$new_s, $delta_x,$delta_y, $bit) = @$state;
    my $from = $state_to_name->($state);
    next if $seen{$from}++;

    print "$from\n";
    if ($old_s == $new_s && $delta_x == 0 && $delta_y == 0) {
      print "  is accepting\n";
      $accepting{$from} = "same";
    }

    foreach my $bit_above (0,1) {
      ($old_s,$new_s, $delta_x,$delta_y) = @$state;
      my $delta = Math::Complex->new($delta_x,$delta_y);
      if ($bit && $bit_above) { $old_s = - $old_s; }

      my $z = $bit_above*$b + $bit + $delta;
      if (($z->Re % 2) == 0 && ($z->Im % 2) == 1) {
        $new_s = - $new_s;
      }
      my $new_bit = ($z->Re + $z->Im) % 2;
      $delta += $bit*$old_s - $new_bit*$new_s;
      $delta /= $b;
      $delta = ~$delta;
      $delta_x = $delta->Re;
      $delta_y = $delta->Im;

      my $new_state = [$old_s,$new_s, $delta_x,$delta_y, $bit_above];
      my $to = $state_to_name->($new_state);
      $table{$from}->{$bit_above} = $to;
      print "  $bit_above -> $to\n";
      if(! $allow_delta{"$delta_x,$delta_y"}) {
        print "bits $bit_above $bit  unexpected delta $delta_x,$delta_y\n";
        print "from  $from\n";
        print "to    $to\n";
        die;
      }
      push @pending, $new_state;
    }
  }

  my $fsm = MyFSM->new(table => \%table,
                       initial => $initial,
                       accepting => \%accepting,
                       title => 'N Other Transitions',
                       flow => 'west',
                      );

  # my @accepting = keys %accepting;
  # foreach my $i (1 .. $#accepting) {
  #   $fsm->merge_state($accepting[0],$accepting[1]);
  # }

  $fsm->simplify (verbose => 1);

  # Reversal fairly wild.
  # $fsm = $fsm->reverse;
  # $fsm->renumber_states;
  # $fsm->simplify (verbose => 1);
  # $fsm->renumber_states;

  $fsm->view;
  exit 0;
}
