#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Create web page sample picture image.png.

use 5.006;
use strict;
use warnings;
use FindBin;
use File::chdir;
use File::stat;
use File::Slurp;
use File::Temp;
use IPC::Run;
use List::Util 'max','min';
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()


my $level = 8;
my $chamfer = .23;
my $AlphaBits = 4;   # 1,2,4

my $version = 0;
foreach my $arg (@ARGV) {
  if ($arg =~ /^VERSION=(.*)/) {
    $version = $1;
  }
}

my $program_filename = File::Spec->catfile($FindBin::Bin,
                                           $FindBin::Script);
my $png_filename = File::Spec->catfile($FindBin::Bin,
                                       File::Spec->updir,
                                       'image.png');

my $temp = File::Temp->newdir;
my $tempdir = $temp->dirname;
local $CWD = $tempdir;

open my $tex_fh, '>', 'image.tex' or die;

print $tex_fh <<'HERE';
\documentclass[a4paper]{article}
\pagestyle{empty}
\usepackage{tikz}
\begin{document}
\begin{tikzpicture}[scale=.263]
  \filldraw ( 0, 0) circle (.1);
  \filldraw (16, 0) circle (.1);
  \node (start-text) at ( 0, 0) [below=.5ex,font=\scriptsize] {start};
  \node (end-text)   at (16, 0) [below=.5ex,font=\scriptsize] {end};
  \draw
HERE

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $join = '';
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);

  if ($n != $n_lo && $n != $n_hi) {
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
    my $sx = $x - $chamfer*$prev_dx;
    my $sy = $y - $chamfer*$prev_dy;
    my $tx = $x + $chamfer*$dx;
    my $ty = $y + $chamfer*$dy;
    print $tex_fh "  $join($sx,$sy)--($tx,$ty)\n";
  } else {
    print $tex_fh "  $join($x,$y)\n";
  }
  $join = '--';
}

print $tex_fh <<'HERE';
;
\end{tikzpicture}
\end{document}
HERE
close $tex_fh or die;
IPC::Run::run(['cat','image.tex']);

IPC::Run::run(['latexmk','-pdf','image.tex'], '<',File::Spec->devnull);

IPC::Run::run(['pdfcrop',
               '--verbose',
               '--margins','10',
               'image.pdf',
               'image-cropped.pdf'],
              '<', File::Spec->devnull)
  or die;
rename 'image-cropped.pdf', 'image.pdf' or die;

print "gs\n";
IPC::Run::run(['gs',
               '-q', # quiet, cf -dQUIET
               '-dNOPAUSE',
               '-r100',
               '-sDEVICE=pnggray',
               "-dTextAlphaBits=$AlphaBits",     # grey scale edge blurring
               "-dGraphicsAlphaBits=$AlphaBits",
               "-sOutputFile=$png_filename",
               'image.pdf'],
              '<', File::Spec->devnull)
  or die;
print "\n";

# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
my $stat = File::stat::stat($program_filename)
  or die "Oops, cannot stat $program_filename: $!";
my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                    localtime($stat->mtime));

my $copyright = File::Slurp::read_file($program_filename);
$copyright =~ /^# (Copyright.*)/m or die;
$copyright = $1;

pngtextadd($png_filename, 'Author',    'Kevin Ryde');
pngtextadd($png_filename, 'Generator', "tools/$FindBin::Script");
pngtextadd($png_filename, 'Title',     'Iterations of the Alternate Paperfolding Curve - sample image');
pngtextadd($png_filename, 'Creation Time', $creation_time);
pngtextadd($png_filename, 'Copyright', $copyright);
pngtextadd($png_filename, 'Homepage',  'http://user42.tuxfamily.org/alternate/index.html');
# pngtextadd($png_filename, 'Version',   $version);

print "uncompressed ",-s $png_filename,"\n";
IPC::Run::run(['optipng','-quiet','-o5',$png_filename]);
print "compressed   ",-s $png_filename,"\n";


# IPC::Run::run(['xpdf','image.pdf']);
# IPC::Run::run(['xzgv',$png_filename]);

exit 0;

# add text to the png image in $png_filename
sub pngtextadd {
  my ($filename, $keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", $filename) == 0
    or die "system(pngtextadd)";
}
